#!/bin/sh

#
#Startup script for gateway
#

#
# PID file
#
PID=LAST_PID

#
# Startup log
#
STARTLOG=logs/startup.log


#Installation Directory
#
dir=`dirname $0`
if [ "$dir" != "." ]
then
  INST=`dirname $dir`
else
  pwd | grep -e 'bin$' > /dev/null
  if [ $? = 0 ]
  then
    INST=".."
  else
    INST=`dirname $dir`
  fi
fi

INST=${INST:-.}

cd $INST

#
# check whether the server might be already running
#
if [ -e $PID ] 
 then 
  if [ -d /proc/$(cat $PID) ]
   then
     echo "A Gateway instance may be already running with process id "$(cat $PID)
     echo "If this is not the case, delete the file $INST/$PID and re-run this script"
     exit 1
   fi
fi


#
#Java command 
#
JAVA=java


#
#Options to the Java VM
#

#set this one if you have ssl problems and need debug info
#OPTS=$OPTS" -Djavax.net.debug=ssl,handshake"

#logging configuration file
OPTS=$OPTS" -Djava.util.logging.config.file=conf/logging.properties"
OPTS=$OPTS" -Dlog4j.configuration=conf/logging.properties"

#
#enable JMX (use jconsole to connect)
#
#OPTS=$OPTS" -Dcom.sun.management.jmxremote"
#enable JMX remote access protected by username/password
#OPTS=$OPTS" -Dcom.sun.management.jmxremote.port=9132 -Dcom.sun.management.jmxremote.authenticate=true"
#OPTS=$OPTS" -Dcom.sun.management.jmxremote.ssl=false"
#OPTS=$OPTS" -Dcom.sun.management.jmxremote.password.file=conf/jmxremote.password"

#
# CRL checker config file
OPTS=$OPTS" -Dcrlmanager.properties.file=conf/crlcheck.properties"

#
#Memory for the VM
#
MEM=-Xmx128m

#
#put all jars in lib/ on the classpath
#
CLASSPATH=.$(find "lib" -name *.jar -exec printf ":{}" \;)
export CLASSPATH

#
# Parameters, such as config file locations
#
#PARAM=<gwProperties> <connectionProperties> <securityProperties>

#
#go
#

nohup $JAVA ${MEM} ${OPTS} ${DEFS} eu.unicore.gateway.Gateway ${PARAM} > $STARTLOG 2>&1  &
echo $! > $PID

echo "UNICORE gateway starting."


