package eu.unicore.gateway.base;

import java.util.Enumeration;

import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;

import org.mortbay.servlet.GzipFilter;

import eu.unicore.gateway.properties.JettyProperties;

/**
 * wrapper around Jetty's {@link GzipFilter} that allows configuration
 * using gateway.properties <br/>
 * 
 * The following configuration options exist.
 * 
 * <ul>
 * <li>gateway.jetty.gzip.minGzipSize  The minimum size of a data chunk that is gzipped (default: 65535</li>
 * <li>gateway.jetty.gzip.bufferSize  The size of the buffer used for gzipping (default: 8192)</li>
 * </ul>
 * 
 * @author schuller
 */
public class ConfigurableGzipFilter extends GzipFilter
{	
	private JettyProperties properties;
	
	public ConfigurableGzipFilter(JettyProperties properties)
	{
		this.properties = properties;
	}
	
	public void init(FilterConfig filterConfig) throws ServletException{
		_minGzipSize=65535;
		super.init(new GatewayFilterConfig(filterConfig));
	}

	class GatewayFilterConfig implements FilterConfig{
		private FilterConfig config;
		
		public GatewayFilterConfig(FilterConfig config){
			this.config=config;
		}
		public String getFilterName() {
			return config.getFilterName();
		}
		public String getInitParameter(String name) {
			String val=config.getInitParameter(name);
			if(val==null)
				val=properties.getGzipProperty(name);
			return val;
		}
		public Enumeration<?> getInitParameterNames() {
			return config.getInitParameterNames();
		}
		public ServletContext getServletContext() {
			return config.getServletContext();
		}
		
	}
	
}
