package eu.unicore.gateway.base;

import org.apache.log4j.Logger;

import eu.unicore.gateway.util.LogUtil;


/**
 * 
 * 
 * Factory class to return {@link IProxyCertificateValidator} implementation.
 * 
 *  
 * @author shahbaz memon, shiraz memon
 *
 */
public class ProxyValidatorFactory {
	
	private static final Logger LOG = LogUtil.getLogger(LogUtil.SECURITY,ProxyValidatorFactory.class);

	/**
	 * 
	 * This methods returns an instance of the proxy validator implementation 
	 * by indirectly referring to the gateway.properties:proxyValidator property.  
	 * 
	 * @param className
	 * @return {@link IProxyCertificateValidator} instance
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 */
	public static IProxyCertificateValidator getProxyValidatorInstance(String className){
		IProxyCertificateValidator proxy = null;
		try {
			proxy = (IProxyCertificateValidator)Class.forName(className).newInstance();
		} catch (InstantiationException e) {
			LogUtil.logException("Cannot instantiate the '"+className+"' proxy validator class.",e,LOG);
		} catch (IllegalAccessException e) {
			LogUtil.logException("Cannot get access to the '"+className+"' proxy validator class.",e,LOG);
		} catch (ClassNotFoundException e) {
			LogUtil.logException("Couldn't find the '"+className+"' proxy validator class.",e,LOG);
		}
		return proxy;
	}

}
