package eu.unicore.gateway.base;

import java.io.Reader;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;

import org.codehaus.stax2.XMLInputFactory2;

import eu.unicore.gateway.soap.SoapVersion;
import eu.unicore.gateway.util.BufferingProxyReader;
import eu.unicore.gateway.util.LogUtil;

/**
 * Holds information during the processing of a single request.
 * This version uses raw Reader and Writer objects and also provides an XMLEventReader.
 * <p>
 * The usage is non intuitive: first data may be read from the XML reader. The characters which are 
 * consumed by the XML reader are recorded (internally in the reader). At any point of time
 * this class user may stop reading the XML events, and start reading from the Reader. The char data
 * processed as XML previously may be replied, and the rest of data may be then read normally.
 * 
 * @see BufferingProxyReader
 * @author golbi
 * @author roger
 */
public class RawMessageExchange 
{
	public final static String X509 = "cert";	
	public final static String REMOTE_IP = "peer";
	public final static String SOAP_ACTION = "soapaction";
	public final static String CONTENT_TYPE= "Content-type";

	private BufferingProxyReader reader;
	private Writer writer;
	private boolean headerPresent;
	
	private XMLEventReader eventReader;
	private String requestUrl;
	private String wsaToAddress;
	private String wsaAction;
	private SoapVersion soapVersion;
	private Map<String,Object> parameterMap=new HashMap<String, Object>();;
	private String destination;
	
	private static XMLInputFactory inFact = XMLInputFactory2.newInstance();
	
	public RawMessageExchange(Reader r, Writer w, int maxHeader) throws XMLStreamException
	{
		reader = new BufferingProxyReader(r, LogUtil.getLogger("gateway.trafficdump", 
			RawMessageExchange.class), maxHeader);
		writer = w;
		setHeaderPresent(false);
		eventReader = inFact.createXMLEventReader(reader);
	}


	public Object getProperty(String key){
		return parameterMap.get(key);
	}

	public void setProperty(String key,Object value){
		parameterMap.put(key, value);
	}

	public String getRequestURL()
	{
		return requestUrl;
	}

	public void setRequestURL(String requesturl)
	{
		this.requestUrl = requesturl;
	}

	public Map<String,Object> getParameterMap()
	{
		return parameterMap;
	}

	public void setParameterMap(Map<String,Object> parametermap)
	{
		this.parameterMap = parametermap;
	}

	public String getDestination() {
		return destination;
	}

	public void setDestination(String destination) {
		this.destination = destination;
	}

	public QName getSOAPHeaderName(){
		return getSoapVersion().getHeader();
	}
	
	public QName getSOAPFaultName(){
		return getSoapVersion().getFault();
	}
	
	public SoapVersion getSoapVersion() {
		return soapVersion;
	}

	public void setSoapVersion(SoapVersion soapVersion) {
		this.soapVersion = soapVersion;
	}

	public String getWsaToAddress() {
		return wsaToAddress;
	}

	public void setWsaToAddress(String wsaToAddress) {
		this.wsaToAddress = wsaToAddress;
	}

	public String getWsaAction() {
		return wsaAction;
	}

	public void setWsaAction(String wsaAction) {
		this.wsaAction = wsaAction;
	}

	public XMLEventReader getEventReader() {
		return eventReader;
	}

	public BufferingProxyReader getReader()
	{
		return reader;
	}

	public Writer getWriter()
	{
		return writer;
	}

	public void setHeaderPresent(boolean headerPresent)
	{
		this.headerPresent = headerPresent;
	}

	public boolean isHeaderPresent()
	{
		return headerPresent;
	}
}
