/**
 * Copyright (c) 2005, Forschungszentrum Juelich
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met: 
 * 
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the Forschungszentrum Juelich nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package eu.unicore.gateway.properties;

import java.io.File;
import java.util.Iterator;



public class ConnectionsProperties extends RProperties
{
	public static final File FILE_CONNECTIONS_PROPERTIES = new File(
		"conf" + File.separator + "connections.properties");
	private File connectionsFile;
	private long lastAccess;

	public ConnectionsProperties(File f)
	{
		super(f, null);
		connectionsFile = f;
	}
	
	private boolean hasChanged()
	{
		long fileMod = connectionsFile.lastModified();
		boolean ret = (lastAccess==0 || lastAccess<fileMod);
		lastAccess=fileMod;
		return ret;
	}
	
	public boolean reloadIfChanged()
	{
		if (hasChanged())
		{
			clearCache();
			loadProperties(connectionsFile);
			return true;
		}
		return false;
	}
	
	public Iterator<Object> getEntries()
	{
		return properties.keySet().iterator();
	}
	
	public String getSite(String name)
	{
		return properties.getProperty(name, null);
	}
}
