package eu.unicore.gateway.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.log4j.Logger;
import org.mortbay.io.EndPoint;
import org.mortbay.io.bio.SocketEndPoint;
import org.mortbay.jetty.Connector;
import org.mortbay.jetty.HttpConnection;
import org.mortbay.jetty.Server;

import eu.unicore.gateway.properties.GatewayProperties;

/**
 * support HTTP "Upgrade" and "101 Switching protocols" mechanism for switching the protocol
 *  
 * @author schuller
 */
public class CustomHttpConnection extends HttpConnection {

	private static final Logger logger=LogUtil.getLogger(LogUtil.GATEWAY,CustomHttpConnection.class);
	private GatewayProperties props;
	
	public CustomHttpConnection(Connector connector, EndPoint endpoint, Server server, 
		GatewayProperties props) 
	{
		super(connector, endpoint, server);
		this.props = props;
	}

	@Override
	protected void handleRequest() throws IOException {
		super.handleRequest();
		String protocol=_response.getHeader("Upgrade");
		if(protocol!=null){
			protocol=_request.getHeader("Upgrade");
			String clazz="";
			try{
				clazz=props.getProtocolPluginHandler(protocol);
				ProtocolPluginHandler handler=(ProtocolPluginHandler)Class.forName(clazz).newInstance();
				handler.configure(_request);
				//OK now take over the connection
				OutputStream os=((SocketEndPoint)getEndPoint()).getOutputStream();
				InputStream is=((SocketEndPoint)getEndPoint()).getInputStream();
				try{
					handler.handle(is,os);
				}catch(IOException io){
					//can't tell the client, but can at least log it
					LogUtil.logException("Error handling custom protocol connection.",io,logger);
				}
				//TODO should we close something?
				this.getConnector().close();
			}catch(Exception ex){
				LogUtil.logException("Error in custom protocol handler <"+clazz+">",ex,logger);
			}
		}
	}

	
	
}
