package eu.unicore.gateway.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.commons.httpclient.HttpConnection;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.methods.ExpectContinueMethod;


/**
 * HTTPClient helper for using the "Update" - "101 Switching protocols" handshake
 * 
 * @author schuller
 */
public class UpdateProtocolMethod extends ExpectContinueMethod {

	private final ProtocolPluginHandler handler;
	
	/**
	 * construct a new method and pass in the custom protocol handler
	 * @param handler
	 */
	public UpdateProtocolMethod(ProtocolPluginHandler handler){
		this.handler=handler;
	}

	/**
	 * override this method only to evade the check 
	 * if the status code is not 100...200
	 */
	@Override
	 protected void readResponse(HttpState state, HttpConnection conn)
	    throws IOException, HttpException {
	        while (this.statusLine == null) {
	            readStatusLine(state, conn);
	            processStatusLine(state, conn);
	            readResponseHeaders(state, conn);
	            processResponseHeaders(state, conn);
	            
	            int status = this.statusLine.getStatusCode();
	            //check for the protocol switch
	            if(status==101){
	            	//OK
	            	return;
	            }
	            //will otherwise retry the call for this status code range
	            if ((status >= 100) && (status < 200)) {
	                this.statusLine = null;
	            }
	        }
	        readResponseBody(state, conn);
	        processResponseBody(state, conn);
	    }

	
	@Override
	public int execute(HttpState state, HttpConnection conn)
	throws HttpException, IOException {
		addRequestHeader("Upgrade", handler.getProtocol());
		addRequestHeader("Connection", "keep-alive");
		try{
			int res=super.execute(state, conn);
			if(res!=101)return res;
		}catch(Throwable t){
			LogUtil.logException("Error executing UpdateProtocol", t);
		}
		OutputStream os=conn.getRequestOutputStream();
		InputStream is=conn.getResponseInputStream();
		handler.handle(is,os);
		return 200;
	}

	@Override
	protected boolean hasRequestContent() {
		return false;
	}

	@Override
	public String getName() {
		return "HEAD";
	}

}
